<?php
namespace BackOfficeOptimizer\Libraries; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use PhpOffice\PhpSpreadsheet\IOFactory;

class ParamPratSheetImporter {
    const TMP_DATA_TABLE = "IMP_PARAMPRATFILIAL";
    const SHEETS_FOLDER = __DIR__ . "/../sheets/";
    protected $HEADER_COLUMNS_MAP = array(
        "IDITIPORESTRICAO" => array(
                                "index"     => 1,
                                "maxLength"    => 9,
                                "nullable"  => false,
                                "zerofill"  => false,
                                "label"     => "Contrato"
                            ),
        "CDFILIAL"          => array(
                                "index"     => 2,
                                "length"    => 4,
                                "nullable"  => false,
                                "zerofill"  => true,
                                "label"     => "Código da Unidade"
                            ),
        "CDSERVFILI"          => array(
                                "index"     => 4,
                                "length"    => 5,
                                "nullable"  => false,
                                "zerofill"  => true,
                                "label"     => "Código do Serviço"
                            )                            
    );
    protected $COLUMNS_MAP = array(
        "CDFILAUXPRAT"      => array(
                                "index"     => 1,
                                "maxLength"    => 7,
                                "nullable"  => false,
                                "zerofill"  => false,
                                "label"     => "Árvore"
                            ),
        "CDPRATO"    => array(
                                "index"     => 2,
                                "nullable"  => false,
                                "zerofill"  => true,
                                "label"     => "Código"
                            ),
        "NMAGRUPGRPRAT" => array(
                                "index"     => 4,
                                "maxLength" => 100,
                                "nullable"  => true,
                                "label"     => "Agrupamentos"
                            ),
        "NMGRUPOPRATO"  => array(
                                "index"     => 5,
                                "maxLength" => 100,
                                "nullable"  => true,
                                "label"     => "Grupo de Pratos"
                            ),
        "IDCOMPLEXIDADE"  => array(
                                "index"     => 6,
                                "maxLength" => 30,
                                "nullable"  => true,
                                "label"     => "Complexidade"
                            ),                             
        "IDPREPREPARO" => array(
                                "index"     => 7,
                                "maxLength" => 4,
                                "nullable"  => true,
                                "label"     => "Pré Peparo"
                            ),
        "IDCOR" => array(
                                "index"     => 8,
                                "maxLength" => 30,
                                "nullable"  => true,
                                "label"     => "Cor"
                            ),
        "NMINGREDIENTEPRINC1" => array(
                                "index"     => 9,
                                "maxLength" => 100,
                                "nullable"  => true,
                                "label"     => "Ing Principal 1"
                            ),
        "NMINGREDIENTEPRINC2" => array(
                                "index"     => 10,
                                "maxLength" => 100,
                                "nullable"  => true,
                                "label"     => "Ing Principal 2"
                            ),   
        "NMAPRERECE" => array(
                                "index"     => 11,
                                "maxLength" => 60,
                                "nullable"  => true,
                                "label"     => "Tipo de Corte"
                            ),
        "NMFORMAPREPARO" => array(
                                "index"     => 12,
                                "maxLength" => 30,
                                "nullable"  => true,
                                "label"     => "Modo Preparo"
                            ),                            
        "NMATRATIVIDADEPRATO" => array(
                                "index"     => 13,
                                "maxLength" => 100,
                                "nullable"  => true,
                                "label"     => "Atratividade"
                            )
    );
    protected $errors = array();
    protected $errorMessages = array();
    protected $nrlogimparampratfi;
    protected $novoCodigo;

    public function __construct($connection, $environment, $novoCodigo) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->COLUMNS_MAP["CDPRATO"]["maxLength"] = $this->getMaxLengthFromMask();
        $this->novoCodigo = $novoCodigo;
    }

    public function getNrlogimparampratfi() {
        return $this->nrlogimparampratfi;
    }

    public function createTemporaryFile($sheetImport) {
        if (    isset($sheetImport) && 
                is_array($sheetImport) && 
                !empty($sheetImport) && 
                isset($sheetImport[0]["b64File"]) && 
                isset($sheetImport[0]["name"])
            ) {
            $sheetB64 = $sheetImport[0]["b64File"];
            $sheetName = $sheetImport[0]["name"];
            $sheet = $this->decodeFromBase64($sheetB64);
            $filePath = self::SHEETS_FOLDER . 'tmp-' . $sheetName;
            file_put_contents($filePath, $sheet);
            chmod($filePath, 0644);
            return $filePath;
        } else {
            throw new \Exception ("sheetUploadFailure");
        }
    }

    public function loadSheet($sheetPath) {
        try {
            $objPHPExcel = IOFactory::load($sheetPath);
            return $objPHPExcel->getSheet(0);
        } catch( \Exception $e) {
            throw new \Exception("invalidSheet");
            
        }
    }

    public function parseSheet($worksheet) {
        try {
            $rows = array();
            foreach ($worksheet->getRowIterator() AS $row) {
                $cellIterator = $row->getCellIterator();
                $cellIterator->resetEnd('N');
                $cellIterator->setIterateOnlyExistingCells(FALSE); // This loops through all cells,
                $cells = [];
                $hasNotNullCell = false;
                foreach ($cellIterator as $cell) {
                    $value = $cell->getValue();
                    $hasNotNullCell = $hasNotNullCell || $value;
                    $cells[] = $value;
                }
                if ($hasNotNullCell) { // At least one cell with value
                    $rows[] = $cells;
                }
            }
            if (count($rows) < 2) {
                throw new \Exception("emptySheet");
            }
            return $rows;
        } catch( \Excention $e) {
            throw $e;
        }
    }

    public function validateSheetColumns($sheet) {
        if (isset($sheet[0])) {
            $header = $sheet[0];
            if(count($header) == 14){
                foreach ($this->HEADER_COLUMNS_MAP as $column) {
                    if ($header[$column["index"]] != $column["label"]) {
                        throw new \Exception("invalidSheetColumnsLabels");
                    }
                }
                $header = $sheet[2];
                foreach ($this->COLUMNS_MAP as $column) {
                    if ($header[$column["index"]] != $column["label"]) {
                        throw new \Exception("invalidSheetColumnsLabels");
                    }
                }
            } else {
                throw new \Exception("invalidSheetColumnsNumber");
            }
        } else {
            throw new \Exception("invalidSheet");
        }
    }

    public function validateSheetRows(&$sheet, $codFilial, $codServico, $tpContrato, $response) {
        foreach ($sheet as $key => &$row) {
            if ($key > 2) { // Skip header row
                $this->validateEmptyValues($row, $key);
                $this->validateInvalidValues($row, $key);
                
            }
            if($key == 1){
                $this->validateEmptyValues($row, $key, true);
                $this->validateInvalidValues($row, $key, true, $codFilial, $codServico, $tpContrato);
            }
        }
        if (!empty($this->errors)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("sheetErrors", 1);
        }
    }

    private function addError($rowNum, $errorType, $error) {
        if (!isset($this->errors[$rowNum])) {
            $this->errors[$rowNum] = array();
        }
        if (!isset($this->errors[$rowNum][$errorType])) {
            $this->errors[$rowNum][$errorType] = array();
        }
        array_push($this->errors[$rowNum][$errorType], $error);
    }

    private function validateEmptyValues(&$row, $rowNum, $isHeader = false) {
        if($isHeader){
            foreach ($this->HEADER_COLUMNS_MAP as $columnName => $column) {
                if (is_null($row[$column["index"]]) && !$column["nullable"]) {
                    $this->addError($rowNum, "emptyCell", $columnName);
                }
            }            
        }else{
            foreach ($this->COLUMNS_MAP as $columnName => $column) {
                if (is_null($row[$column["index"]]) && !$column["nullable"]) {
                    $this->addError($rowNum, "emptyCell", $columnName);
                }
            }            
        }

    }

    private function getMaxLengthFromMask() {

        $sql = "SELECT CDPICTPRAT, CDPICTPROD, DSDNSCARDOTIMIZADO, IDUTRPADCA FROM PARAMGERAL";
    
        $statement = $this->connection->prepare($sql);
        $statement->execute();
        $paramGeralData =  $statement->fetchAll();
        $paramGeralData = !empty($paramGeralData) ? $paramGeralData[0] : false;
        $cdpictprat = $paramGeralData ? $paramGeralData["CDPICTPRAT"] : false;
        if ($cdpictprat) {
            return strlen(str_replace(".", "", $cdpictprat));
        } else {
            return 10;
        }
    }

    private function validateInvalidValues(&$row, $rowNum, $isHeader = false, $codFilial = null, $codServico = null, $tpContrato = null) {
        if($isHeader){
            foreach ($this->HEADER_COLUMNS_MAP as $columnName => $column) {
                if (strlen($row[$column["index"]]) != 0) {
                    if (isset($column["length"]) && strlen($row[$column["index"]]) != $column["length"]) {
                        if (strlen($row[$column["index"]]) < $column["length"] && isset($column["zerofill"]) && $column["zerofill"]) {
                            $row[$column["index"]] = str_pad($row[$column["index"]], $column["length"], "0", STR_PAD_LEFT);
                        } else {
                            $this->addError($rowNum, "invalidLength", $columnName); 
                        }
                    } else if (isset($column["maxLength"]) && strlen($row[$column["index"]]) > $column["maxLength"]) {
                        $this->addError($rowNum, "invalidMaxLength", $columnName); 
                    } 
                    if ($column["index"] == 1 && $row[$column["index"]] != $tpContrato){
                        $this->addError($rowNum, "invalidTpContrato", $columnName); 
                    }
                    if ($column["index"] == 2 && $row[$column["index"]] != $codFilial){
                        $this->addError($rowNum, "invalidCodFilial", $columnName); 
                    }
                    if ($column["index"] == 4 && $row[$column["index"]] != $codServico){
                        $this->addError($rowNum, "invalidCodServico", $columnName); 
                    }
                }
            }            
        }else{
            foreach ($this->COLUMNS_MAP as $columnName => $column) {
                if (strlen($row[$column["index"]]) != 0) {
                    if (isset($column["length"]) && strlen($row[$column["index"]]) != $column["length"]) {
                        if (strlen($row[$column["index"]]) < $column["length"] && isset($column["zerofill"]) && $column["zerofill"]) {
                            $row[$column["index"]] = str_pad($row[$column["index"]], $column["length"], "0", STR_PAD_LEFT);
                        } else {
                            $this->addError($rowNum, "invalidLength", $columnName); 
                        }
                    } else if (isset($column["maxLength"]) && strlen($row[$column["index"]]) > $column["maxLength"]) {
                        $this->addError($rowNum, "invalidMaxLength", $columnName); 
                    }
                }
            }            
        }
    
    }

    private function decodeErrorMessages() {
        foreach ($this->errors as $rowNum => $error) {
            $message = "Linha: " . ($rowNum + 1) ." - ";
            //array_push($this->errorMessages, $message);
            if($rowNum == 1){
                if (isset($error["invalidTpContrato"])) {
                    foreach (($error["invalidTpContrato"]) as $column) {
                        $messageError = $message . "O " . $this->HEADER_COLUMNS_MAP[$column]["label"] . " é diferente do tipo de contrato selecionado no sistema.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
                if (isset($error["invalidCodFilial"])) {
                    foreach (($error["invalidCodFilial"]) as $column) {
                        $messageError = $message . "O " . $this->HEADER_COLUMNS_MAP[$column]["label"] . " é diferente da unidade selecionada no sistema.";
                        array_push($this->errorMessages, $messageError);
                    }
                }                
                if (isset($error["invalidCodServico"])) {
                    foreach (($error["invalidCodServico"]) as $column) {
                        $messageError = $message . "O " . $this->HEADER_COLUMNS_MAP[$column]["label"] . " é diferente do serviço selecionado no sistema.";
                        array_push($this->errorMessages, $messageError);
                    }
                }                
                if (isset($error["emptyCell"])) {
                    foreach (($error["emptyCell"]) as $column) {
                        $messageError = $message . "O " . $this->HEADER_COLUMNS_MAP[$column]["label"] . " deve ser informado.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
    
                if (isset($error["invalidLength"])) {
                    foreach (($error["invalidLength"]) as $column) {
                        $messageError = $message . "O " . $this->HEADER_COLUMNS_MAP[$column]["label"] . " deve possuir exatamente ". $this->HEADER_COLUMNS_MAP[$column]["length"] . " caracteres.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
    
                if (isset($error["invalidMaxLength"])) {
                    foreach (($error["invalidMaxLength"]) as $column) {
                        $messageError = $message . "O " . $this->HEADER_COLUMNS_MAP[$column]["label"] . " deve possuir no máximo ". $this->HEADER_COLUMNS_MAP[$column]["maxLength"] . " caracteres.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
    
                if (isset($error["filialNotFound"])) {
                    $messageError = $message . "Unidade não encontrada.";
                    array_push($this->errorMessages, $messageError);
                }                
            }else{
            
                if (isset($error["emptyCell"])) {
                    foreach (($error["emptyCell"]) as $column) {
                        $messageError = $message . "O " . $this->COLUMNS_MAP[$column]["label"] . " deve ser informado.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
    
                if (isset($error["invalidLength"])) {
                    foreach (($error["invalidLength"]) as $column) {
                        $messageError = $message . "O " . $this->COLUMNS_MAP[$column]["label"] . " deve possuir exatamente ". $this->COLUMNS_MAP[$column]["length"] . " caracteres.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
    
                if (isset($error["invalidMaxLength"])) {
                    foreach (($error["invalidMaxLength"]) as $column) {
                        $messageError = $message . "O " . $this->COLUMNS_MAP[$column]["label"] . " deve possuir no máximo ". $this->COLUMNS_MAP[$column]["maxLength"] . " caracteres.";
                        array_push($this->errorMessages, $messageError);
                    }
                }
            }
        }
    }

    private function decodeErrorMessagesFromProcedure($nrlogimparampratfi) {

        $sql = "SELECT DSLOGIMPARAMPRATFI FROM LOG_IMP_PARAMPRATFI WHERE NRLOGIMPARAMPRATFI = :NRLOGIMPARAMPRATFI";
        $queryParams = array(
            "NRLOGIMPARAMPRATFI" => $nrlogimparampratfi,
        );
        $statement = $this->connection->prepare($sql);
        $statement->execute($queryParams);
        $errors =  $statement->fetchAll();
        usort($errors, function($a, $b) {
			return strnatcmp($a['DSLOGIMPARAMPRATFI'], $b['DSLOGIMPARAMPRATFI']);
		});
        foreach ($errors as $error) {
            array_push($this->errorMessages, $error["DSLOGIMPARAMPRATFI"]);
        }
    }

    private function decodeFromBase64($sheet) {
        return base64_decode($sheet, true);
    }

    public function saveSheetData($sheet, $response, $row, $nrorg) {
        $rows = $this->buildRows($sheet, $row, $nrorg);
        return $this->persistData($rows, $response);
    }

    private function buildRows($sheet, $row_param, $nrorg) {
        $nrimpparampratfilial = $this->novoCodigo->getCodeByTableName(self::TMP_DATA_TABLE, $nrorg);
        $rows = array();
        $now = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $cdfilial; 
        $cdservfili;
        $idtiporestricao;
        foreach ($sheet as $key => $sheetRow) {
            if($key == 1){
               $idtiporestricao = $sheetRow[1];
               $cdfilial =  $sheetRow[2];
               $cdservfili = $sheetRow[4];
            }
            if ($key > 2) { // Skip header row
                $row = array(
                    "NRIMPPARAMPRATFILIAL" => $nrimpparampratfilial,
                    "NRINDICE"        => $key + 1,
                    "CDFILIAL"        => $cdfilial,
                    "CDSERVFILI"      => $cdservfili,
                    "IDTIPORESTRICAO" => $idtiporestricao,
                    "CDFILAUXPRAT"        => $sheetRow[$this->COLUMNS_MAP["CDFILAUXPRAT"]["index"]],
                    "CDPRATO"      => $sheetRow[$this->COLUMNS_MAP["CDPRATO"]["index"]],
                    "NMAGRUPGRPRAT"   => $sheetRow[$this->COLUMNS_MAP["NMAGRUPGRPRAT"]["index"]],
                    "NMGRUPOPRATO"   => $sheetRow[$this->COLUMNS_MAP["NMGRUPOPRATO"]["index"]],
                    "IDCOMPLEXIDADE"    => $sheetRow[$this->COLUMNS_MAP["IDCOMPLEXIDADE"]["index"]],
                    "IDPREPREPARO"    => $sheetRow[$this->COLUMNS_MAP["IDPREPREPARO"]["index"]],
                    "IDCOR"         => $sheetRow[$this->COLUMNS_MAP["IDCOR"]["index"]],
                    "NMINGREDIENTEPRINC1"    => $sheetRow[$this->COLUMNS_MAP["NMINGREDIENTEPRINC1"]["index"]],
                    "NMINGREDIENTEPRINC2"    => $sheetRow[$this->COLUMNS_MAP["NMINGREDIENTEPRINC2"]["index"]],
                    "NMAPRERECE"    => $sheetRow[$this->COLUMNS_MAP["NMAPRERECE"]["index"]],
                    "NMFORMAPREPARO"    => $sheetRow[$this->COLUMNS_MAP["NMFORMAPREPARO"]["index"]],
                    "NMATRATIVIDADEPRATO"    => $sheetRow[$this->COLUMNS_MAP["NMATRATIVIDADEPRATO"]["index"]],
                    "NRORG"           => $nrorg,
                    "IDATIVO"         => "S",
                    "DTINCLUSAO"      => $now,
                    "DTULTATU"        => $now,
                    "NRORGINCLUSAO"   => $nrorg,
                    "CDOPERINCLUSAO"  => $row_param['CDOPERADOR'],
                    "NRORGULTATU"     => $nrorg,
                    "CDOPERULTATU"    => $row_param['CDOPERADOR']
                );
                array_push($rows, $row);
            }
        }
        return $rows; 
    }

    private function persistData($rowsToSave, $response) {
        $nrimpparampratfilial = $rowsToSave[0]["NRIMPPARAMPRATFILIAL"];
        $cdfilial = $rowsToSave[0]["CDFILIAL"];
        $cdservfili = $rowsToSave[0]["CDSERVFILI"];
        $idtiporestricao = $rowsToSave[0]["IDTIPORESTRICAO"];
        
        $this->connection->beginTransaction();
        foreach ($rowsToSave as $row) {
            try {
                $this->insertImpgrupoprato($row);
            } catch (\Exception $e) {
                if (strrpos($e->getMessage(), "ORA-02291")) {
                    $this->addError($row["NRINDICE"] - 1, "filialNotFound", true);
                    break;
                } else {
                    throw $e;
                }
            }
        }
        if (!empty($this->errors)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            $this->connection->rollBack();
            throw new \Exception("sheetErrors", 1);
        }
        $this->connection->commit();
        return [
            'nrimpparampratfilial' => $nrimpparampratfilial,
            'cdfilial' => $cdfilial,
            'cdservfili' => $cdservfili,
            'idtiporestricao' => $idtiporestricao
        ];
    }

    private function insertImpgrupoprato($row) {
        $sql = "INSERT INTO IMP_PARAMPRATFILIAL 
                    (NRIMPPARAMPRATFILIAL, NRINDICE, CDFILIAL, CDFILAUXPRAT, CDPRATO, NMAGRUPGRPRAT, NMGRUPOPRATO, IDCOMPLEXIDADE, 
                    IDPREPREPARO, IDCOR, NMINGREDIENTEPRINC1, NMINGREDIENTEPRINC2, NMAPRERECE, NMFORMAPREPARO, NMATRATIVIDADEPRATO, 
                    NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
                VALUES                      
                    (:NRIMPPARAMPRATFILIAL, :NRINDICE, :CDFILIAL, :CDFILAUXPRAT, :CDPRATO, :NMAGRUPGRPRAT, :NMGRUPOPRATO, :IDCOMPLEXIDADE, 
                    :IDPREPREPARO, :IDCOR, :NMINGREDIENTEPRINC1, :NMINGREDIENTEPRINC2, :NMAPRERECE, :NMFORMAPREPARO, :NMATRATIVIDADEPRATO, 
                    :NRORG, :IDATIVO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)
                ";

        $query = $this->connection->prepare($sql);
        $query->bindValue("NRIMPPARAMPRATFILIAL",  $row['NRIMPPARAMPRATFILIAL']);
        $query->bindValue("NRINDICE",              $row['NRINDICE']);
        $query->bindValue("CDFILIAL",              $row['CDFILIAL']);
        $query->bindValue("CDFILAUXPRAT",          $row['CDFILAUXPRAT']);
        $query->bindValue("CDPRATO",               $row['CDPRATO']);
        $query->bindValue("NMAGRUPGRPRAT",         $row['NMAGRUPGRPRAT']);
        $query->bindValue("NMGRUPOPRATO",          $row['NMGRUPOPRATO']);
        $query->bindValue("IDCOMPLEXIDADE",        $row['IDCOMPLEXIDADE']);
        $query->bindValue("IDPREPREPARO",          $row['IDPREPREPARO']);
        $query->bindValue("IDCOR",                 $row['IDCOR']);
        $query->bindValue("NMINGREDIENTEPRINC1",   $row['NMINGREDIENTEPRINC1']);
        $query->bindValue("NMINGREDIENTEPRINC2",   $row['NMINGREDIENTEPRINC2']);
        $query->bindValue("NMAPRERECE",            $row['NMAPRERECE']);
        $query->bindValue("NMFORMAPREPARO",        $row['NMFORMAPREPARO']);
        $query->bindValue("NMATRATIVIDADEPRATO",   $row['NMATRATIVIDADEPRATO']);
        $query->bindValue("NRORG",                 $row['NRORG']);
        $query->bindValue("IDATIVO",               $row['IDATIVO']);
        $query->bindValue("DTINCLUSAO",            $row['DTINCLUSAO']);
        $query->bindValue("DTULTATU",              $row['DTULTATU']);
        $query->bindValue("NRORGINCLUSAO",         $row['NRORGINCLUSAO']);
        $query->bindValue("CDOPERINCLUSAO",        $row['CDOPERINCLUSAO']);
        $query->bindValue("NRORGULTATU",           $row['NRORGULTATU']);
        $query->bindValue("CDOPERULTATU",          $row['CDOPERULTATU']);
        $query->execute();
    }

    public function executeProcedure($nrimpparampratfilial, $cdfilial, $cdservfili, $idtiporestricao, $idutpadca, $response) {
        $procedure = new StoredProcedure($this->connection, 'IMPORTA_PARAMPRATFILIAL_XLS');
        $procedure->addParam(new Param('P_NRIMPPARAMPRATFILIAL', Param::PARAM_INPUT, $nrimpparampratfilial));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $cdservfili));
        $procedure->addParam(new Param('P_IDTIPORESTRICAO', Param::PARAM_INPUT, $idtiporestricao));
        $procedure->addParam(new Param('IDUTRPADCA', Param::PARAM_INPUT, $idutpadca));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, NULL, 20));
        $this->nrlogimparampratfi = $procedure->execute()["P_RETORNO"];
        
        if (!is_null($this->nrlogimparampratfi)) {
            $this->decodeErrorMessagesFromProcedure($this->nrlogimparampratfi);
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("procedureErrors", 1);
        }
    }

    public function removeTemporaryFile($filePath) {
        unlink($filePath);
    }

    public function removeSheetTmpData($nrimpparampratfilial) {
        $sql = "DELETE FROM IMP_PARAMPRATFILIAL WHERE NRIMPPARAMPRATFILIAL = :NRIMPPARAMPRATFILIAL";

        $query = $this->connection->prepare($sql);

        $query->bindValue("NRIMPPARAMPRATFILIAL", $nrimpparampratfilial);
        $query->execute();
    }

    public function removeSheetLog($nrlogimparampratfi) {
        $sql = "DELETE FROM LOG_IMP_PARAMPRATFI WHERE NRLOGIMPARAMPRATFI = :NRLOGIMPARAMPRATFI";

        $query = $this->connection->prepare($sql);

        $query->bindValue("NRLOGIMPARAMPRATFI", $nrlogimparampratfi);
        $query->execute();
    }
}
