<?php

namespace BackOfficeOptimizer\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use BackOfficeOptimizer\Libraries\JWT;

class IntegracaoCardapio extends \Teknisa\Libs\Controller\CRUD
{

    protected $dataSourceName;
    protected $newCode;
    protected $callback;
    const NRORG = 1;
    const REQUEST_PARAMS = array("cardapio", "NRCENARIOCARD");
    const CARDAPIO_PARAMS = array("unidade", "servico", "mes", "operador", "datas");
    const DIA_PARAMS = array("dia", "comensais", "pratos");
    const PRATO_PARAMS = array("prato", "nrPratoCardapio", "unidadePrato", "percentualConsumo", "tipoPrato");

    function __construct($crudService, $environment, $newCode, $newCodeNT, $connection/* , $sqlLogger */)
    {
        parent::__construct($crudService, $environment);

        $this->newCode = $newCode;
        $this->newCodeNT = $newCodeNT;
        $this->connection = $connection;
        // $this->sqlLogger = $sqlLogger;
    }

    function calOptServer(DTO\Request $request, DTO\Response $response)
    {
        $token = isset($request->getParameters()['$token'])? $request->getParameters()['$token']: null;
        if($token){
            $userInfo = JWT::getTokenData($token);
        }
        $nrorg = $userInfo['NRORG'] ? $userInfo['NRORG'] : 1;
        $host = $_SERVER['HTTP_HOST'];
        $reqData = $request->getRow();
        $url = $reqData["url"];
        $body = $reqData["cloneData"];
        
        set_time_limit(320000); //600
        ini_set('post_max_size', '2048M');
        ini_set('upload_max_filesize', '2048M');
        ini_set('memory_limit', '2048M');
        $header = array();

        $header[] = 'Content-Type: application/json';
        $header[] = 'X-Custom-Header: ' . $host .' - ' . $nrorg;
        $header[] = 'Content-Length: ' . strlen(json_encode($body));
        

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($body));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 320000); //10
        curl_setopt($ch, CURLOPT_TIMEOUT, 320000); //600

        $resultStr = curl_exec($ch);

        //$result = json_decode($resultStr, true);
        $result = array($resultStr);
        
        $error = curl_error($ch);
        curl_close($ch);

        switch (json_last_error()) {
            case JSON_ERROR_NONE:
                break;
            case JSON_ERROR_DEPTH:
                $decodeError = 'Maximum stack depth exceeded';
                break;
            case JSON_ERROR_STATE_MISMATCH:
                $decodeError = 'Underflow or the modes mismatch';
                break;
            case JSON_ERROR_CTRL_CHAR:
                $decodeError = 'Unexpected control character found';
                break;
            case JSON_ERROR_SYNTAX:
                $decodeError = 'Syntax error, malformed JSON';
                break;
            case JSON_ERROR_UTF8:
                $decodeError = 'Malformed UTF-8 characters, possibly incorrectly encoded';
                break;
            default:
                $decodeError = 'Unknown error';
                break;
        }

        if (isset($result["error"])) {
            var_dump($result);
            die;
            throw new \Exception($result["error"]);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($decodeError)) {
            throw new \Exception($decodeError . ": " . $resultStr);
        } else {
            $response->addDataSet(new DataSet('cardIntStatus', $result));
        }
    }

    function integracaoCardapio(DTO\Request $request, DTO\Response $response)
    {
        try {
            // $this->sqlLogger->setLogRoute("/integracaoCardapio");
            // $this->connection->getConfiguration()->setSQLLogger($this->sqlLogger);
            $parameters = $request->getParameters()['row'];
            // var_dump($parameters);die;
            $this->callback = $parameters;
            $this->callback["hasErrors"] = false;
            $nrlogintcardames = $this->newCode->getCodeByTableName("LOGINTCARDAMES", self::NRORG);
            $this->saveLogintcardames($nrlogintcardames, $parameters);

            if (!$this->callback["hasErrors"]) {
                $datas = $parameters["cardapio"]["datas"];
                $NRCENARIOCARD = $parameters["NRCENARIOCARD"];
                if (is_array($datas)) {
                    $pratoLength = $this->getPratMaskLength();
                    foreach ($datas as $key => $dia) {
                        $this->saveLogintcardadia($nrlogintcardames, $dia, $key, $pratoLength);
                    }
                } else {
                    $this->callback["hasErrors"] = true;
                    $this->callback["cardapio"]["erros"] = array();
                    $erro = "A propriedade 'datas' deve ser um array.";
                    array_push($this->callback["cardapio"]["erros"], $erro);
                }
                if (!$this->callback["hasErrors"]) {
                    $this->connection->beginTransaction();
                    $p_erros = $this->executeProcedure($nrlogintcardames, $NRCENARIOCARD);
                    $this->connection->commit();
                    if ($p_erros['P_IDERRO'] == "S") {
                        $this->callback["hasErrors"] = true;
                        $this->setErrorsFromProcedure($nrlogintcardames);
                    }
                }
            }
            $this->handleCallback($response);
        } catch (\Exception $e) {
            // var_dump('Erro durante a integracaoCardapio, podem ser os parametros.');
            var_dump($e);
            error_log($e);
            throw $e;
        }
    }

    function saveLogintcardames($nrlogintcardames, $params)
    {
        $sql = "INSERT INTO LOGINTCARDAMES (NRLOGINTCARDAMES, DTLOGINT, CDFILIAL, CDSERVFILI, DTMESCARD, CDOPERADOR, DSARQIMPORT) VALUES (:NRLOGINTCARDAMES, SYSDATE, :CDFILIAL, :CDSERVFILI, :DTMESCARD, :CDOPERADOR, :DSARQIMPORT)";
        $currentTime = new \DateTime();
        $currentTime = $currentTime->format("d/m/Y H:i:s");
        $diff = array_diff(self::REQUEST_PARAMS, array_keys($params));
        if (empty($diff)) {
            $diff = array_diff(self::CARDAPIO_PARAMS, array_keys($params["cardapio"]));
            if (empty($diff)) {
                $cardapio = $params["cardapio"];
                $queryParameters = array(
                    "NRLOGINTCARDAMES" => $nrlogintcardames,
                    // "DTLOGINT" => $currentTime,
                    "CDFILIAL" => $cardapio["unidade"],
                    "CDSERVFILI" => $cardapio["servico"],
                    "DTMESCARD" => $cardapio["mes"],
                    "CDOPERADOR" => $cardapio["operador"],
                    "DSARQIMPORT" => json_encode($params),
                );
            } else {
                $erro = "Parâmetros faltando na requisição. (" . implode(", ", $diff) . ")";
                $this->callback["hasErrors"] = true;
                $this->callback["cardapio"]["erros"] = array();
                array_push($this->callback["cardapio"]["erros"], $erro);
                $cardapio = $params["cardapio"];
                $queryParameters = array(
                    "NRLOGINTCARDAMES" => $nrlogintcardames,
                    "DTLOGINT" => $currentTime,
                    "CDFILIAL" => isset($cardapio["unidade"]) ? $cardapio["unidade"] : null,
                    "CDSERVFILI" => isset($cardapio["servico"]) ? $cardapio["servico"] : null,
                    "DTMESCARD" => isset($cardapio["mes"]) ? $cardapio["mes"] : null,
                    "CDOPERADOR" => isset($cardapio["operador"]) ? $cardapio["operador"] : null,
                    "DSARQIMPORT" => json_encode($params),
                );
            }
        } else {
            $erro = "Parâmetros faltando na requisição. (" . implode(", ", $diff) . ")";
            $this->callback["hasErrors"] = true;
            $this->callback["erros"] = array();
            array_push($this->callback["erros"], $erro);
            $queryParameters = array(
                "NRLOGINTCARDAMES" => $nrlogintcardames,
                "DTLOGINT" => $currentTime,
                "CDFILIAL" => null,
                "CDSERVFILI" => null,
                "DTMESCARD" => null,
                "CDOPERADOR" => null,
                "DSARQIMPORT" => json_encode($params),
            );
        }
        $query = $this->connection->prepare($sql);
        $query->execute($queryParameters);
    }

    function saveLogintcardadia($nrlogintcardames, $dia, $indexDia, $pratoLength)
    {
        $sql = "INSERT INTO LOGINTCARDADIA (NRLOGINTCARDADIA, NRLOGINTCARDAMES, DTCARDAPIO, QTCOMESTICAR, QTCLIESTICAR, QTFUNESTICAR, QTDOAESTICAR) VALUES (:NRLOGINTCARDADIA, :NRLOGINTCARDAMES, :DTCARDAPIO, :QTCOMESTICAR, :QTCLIESTICAR, :QTFUNESTICAR, :QTDOAESTICAR)";
        $diff = array_diff(self::DIA_PARAMS, array_keys($dia));
        if (empty($diff)) {
            $nrlogintcardadia = $this->newCodeNT->getCodeByTableName("LOGINTCARDADIA", self::NRORG);
            $queryParameters = array(
                "NRLOGINTCARDAMES" => $nrlogintcardames,
                "NRLOGINTCARDADIA" => $nrlogintcardadia,
                "DTCARDAPIO" => $dia["dia"],
                "QTCOMESTICAR" => $dia["comensais"],
                "QTCLIESTICAR" => $dia["comenCLI"],
                "QTFUNESTICAR" => $dia["comenFUN"],
                "QTDOAESTICAR" => $dia["comenDOA"],
            );
            $query = $this->connection->prepare($sql);
            $query->execute($queryParameters);
            if (is_array($dia["pratos"])) {
                $pratos = $dia["pratos"];
                  foreach ($pratos as $key => $prato) {
                    $this->saveLogintcompcard($nrlogintcardames, $nrlogintcardadia, $indexDia, $prato, $key, $pratoLength);
                }
            } else {
                $this->callback["hasErrors"] = true;
                $this->callback["cardapio"]["datas"][$indexDia]["erros"] = array();
                $erro = "A propriedade 'pratos' deve ser um array.";
                array_push($this->callback["cardapio"]["datas"][$indexDia]["erros"], $erro);
            }
        } else {
            $erro = "Parâmetros faltando na requisição. (" . implode(", ", $diff) . ")";
            $this->callback["hasErrors"] = true;
            $this->callback["cardapio"]["datas"][$indexDia]["erros"] = array();
            array_push($this->callback["cardapio"]["datas"][$indexDia]["erros"], $erro);
        }
    }

    function saveLogintcompcard($nrlogintcardames, $nrlogintcardadia, $indexDia, $prato, $indexPrato, $pratoLength)
    {
        $sql = "INSERT INTO LOGINTCOMPCARD (NRLOGINTCOMPCARD, NRLOGINTCARDAMES, NRLOGINTCARDADIA, CDFILAUXPRAT, CDPRATO, NRSEQPRATCAR, CDTIPOPRAT, VRPECONSCARD) VALUES (:NRLOGINTCOMPCARD, :NRLOGINTCARDAMES, :NRLOGINTCARDADIA, :CDFILAUXPRAT, :CDPRATO, :NRSEQPRATCAR, :CDTIPOPRAT, :VRPECONSCARD)";
        
        $prato["tipoPrato"] = $prato["COD_TIP_PRATO"];
        $prato["nrPratoCardapio"] = $prato["sequencial"];
        if((substr($prato["prato"], 0, 1) == 'P') && (strlen($prato["prato"]) == $pratoLength + 1)){
           $prato["unidadePrato"] = 'P';
           $prato["prato"] = substr($prato["prato"], 1);
        } else if(strlen($prato["prato"]) > $pratoLength + 1){
            $prato["unidadePrato"] = substr($prato["prato"], 0, 4);
            $prato["prato"] = substr($prato["prato"], 4);
        }
        $diff = array_diff(self::PRATO_PARAMS, array_keys($prato));
        
        if (empty($diff)) {
            $nrlogintcompcard = $this->newCodeNT->getCodeByTableName("LOGINTCOMPCARD", self::NRORG);
            $queryParameters = array(
                "NRLOGINTCOMPCARD" => $nrlogintcompcard,
                "NRLOGINTCARDAMES" => $nrlogintcardames,
                "NRLOGINTCARDADIA" => $nrlogintcardadia,
                "CDPRATO" => $prato["prato"],
                "CDFILAUXPRAT" => $prato["unidadePrato"],
                "NRSEQPRATCAR" => $prato["nrPratoCardapio"],
                "CDTIPOPRAT" => $prato["tipoPrato"],
                "VRPECONSCARD" => $prato["percentualConsumo"],
            );
            $query = $this->connection->prepare($sql);
            $query->execute($queryParameters);
        } else {
            $erro = "Parâmetros faltando na requisição. (" . implode(", ", $diff) . ")";
            $this->callback["hasErrors"] = true;
            $this->callback["cardapio"]["datas"][$indexDia]["pratos"][$indexPrato]["erros"] = array();
            array_push($this->callback["cardapio"]["datas"][$indexDia]["pratos"][$indexPrato]["erros"], $erro);
        }
    }

    function getPratMaskLength()
    {
        $sql = "SELECT CDPICTPRAT FROM PARAMGERAL WHERE CDPARGERAL = '0001'";
        $query = $this->connection->prepare($sql);
        $query->execute();
        $result = $query->fetch();
        return strlen(str_replace('.', '', $result['CDPICTPRAT']));
    }


    function executeProcedure($nrlogintcardmes, $NRCENARIOCARD)
    {
        $procedure = new StoredProcedure($this->connection, 'INTEGRA_CARDAPIO_WEBSERVICE');
        $procedure->addParam(new Param('P_NRLOGINTCARDMES', Param::PARAM_INPUT, $nrlogintcardmes, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDERRO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRCENARIOCARD', Param::PARAM_INPUT, $NRCENARIOCARD, Param::PARAM_TYPE_STR, 10));
        return $procedure->execute();
    }

    function buildFilterForRow($procedureErrors)
    {
        return array(
            "NRLOGINTCARDAMES" => $procedureErrors['NRLOGINTCARDAMES'], // general error
            "NRLOGINTCARDADIA" => $procedureErrors['NRLOGINTCARDADIA'], // day error
            "NRLOGINTCOMPCARD" => $procedureErrors['NRLOGINTCOMPCARD'], // plate error
        );
    }

    function getErrorsFromProcedure($nrlogintcardames)
    {
        $sql = "SELECT * FROM LOGINTCARDWS WHERE NRLOGINTCARDAMES = :NRLOGINTCARDAMES";
        $queryParameters = array(
            "NRLOGINTCARDAMES" => $nrlogintcardames,
        );
        $query = $this->connection->prepare($sql);
        $query->execute($queryParameters);
        return $query->fetchAll();
    }

    function getDayFromLogintcardadia($NRLOGINTCARDADIA)
    {
        $sql = "SELECT TRUNC(DTCARDAPIO) AS DTCARDAPIO FROM LOGINTCARDADIA WHERE NRLOGINTCARDADIA = :NRLOGINTCARDADIA";
        $queryParameters = array(
            "NRLOGINTCARDADIA" => $NRLOGINTCARDADIA,
        );
        $query = $this->connection->prepare($sql);
        $query->execute($queryParameters);
        return $query->fetchAll()[0];
    }

    function getPlateFromLogintcompcard($NRLOGINTCOMPCARD)
    {
        $sql = "SELECT LICD.DTCARDAPIO AS DTCARDAPIO , LICC.CDPRATO AS CDPRATO , LICC.CDTIPOPRAT AS CDTIPOPRAT , LICC.CDFILAUXPRAT AS CDFILAUXPRAT FROM LOGINTCOMPCARD LICC INNER JOIN LOGINTCARDADIA LICD ON LICD.NRLOGINTCARDADIA = LICC.NRLOGINTCARDADIA WHERE NRLOGINTCOMPCARD = :NRLOGINTCOMPCARD";
        $queryParameters = array(
            "NRLOGINTCOMPCARD" => $NRLOGINTCOMPCARD,
        );
        $query = $this->connection->prepare($sql);
        $query->execute($queryParameters);
        return $query->fetchAll()[0];
    }

    function setGeneralErrorOnCallback($curProError)
    {
        $this->callback['cardapio']['errors'][] = $curProError['DSITEM'] . ' - ' . $curProError['DSERRO'];
    }

    function setDayErrorOnCallback($curProError, $NRLOGINTCARDADIA)
    {
        $dayWithError = $this->getDayFromLogintcardadia($NRLOGINTCARDADIA);
        for ($i = 0; $i < sizeof($this->callback['cardapio']['datas']); $i++) {
            $currData = $this->callback['cardapio']['datas'][$i];

            if ($currData['dia'] == str_replace(' 00:00:00', '', $dayWithError['DTCARDAPIO'])) {
                $this->callback['cardapio']['datas'][$i]['errors'][] = $curProError['DSITEM'] . ' - ' . $curProError['DSERRO'];
            }
        }
    }
    function setPlateErrorOnCallback($curProError, $NRLOGINTCOMPCARD)
    {
        $plateWithError = $this->getPlateFromLogintcompcard($NRLOGINTCOMPCARD);

        for ($i = 0; $i < sizeof($this->callback['cardapio']['datas']); $i++) {
            $currData = $this->callback['cardapio']['datas'][$i];
            for ($j = 0; $j < sizeof($currData['pratos']); $j++) {
                $currPlate = $currData['pratos'][$j];
                if ($currData['dia'] == str_replace(' 00:00:00', '', $plateWithError['DTCARDAPIO']) && $currPlate['prato'] == $plateWithError['CDPRATO'] && $currPlate['tipoPrato'] == $plateWithError['CDTIPOPRAT'] && $currPlate['unidadePrato'] == $plateWithError['CDFILAUXPRAT']) {
                    $this->callback['cardapio']['datas'][$i]['pratos'][$j]['errors'][] = $curProError['DSITEM'] . ' - ' . $curProError['DSERRO'];
                }
            }
        }
    }

    function setErrorsFromProcedure($nrlogintcardames)
    {
        $procedureErrors = $this->getErrorsFromProcedure($nrlogintcardames);

        if (!empty($procedureErrors) && is_array($procedureErrors)) {
            for ($i = 0; $i < sizeof($procedureErrors); $i++) {
                $curProError = $procedureErrors[$i];
                $flowDivider = $this->buildFilterForRow($curProError);
                if ($flowDivider['NRLOGINTCARDADIA'] == null && $flowDivider['NRLOGINTCOMPCARD'] == null) {
                    $this->setGeneralErrorOnCallback($curProError); // General Error
                } else if ($flowDivider['NRLOGINTCARDADIA'] != null) {
                    $this->setDayErrorOnCallback($curProError, $flowDivider['NRLOGINTCARDADIA']); // Day Error
                } else if ($flowDivider['NRLOGINTCOMPCARD'] != null) {
                    $this->setPlateErrorOnCallback($curProError, $flowDivider['NRLOGINTCOMPCARD']); // Plate Error
                }
            }
        }
    }

    function handleCallback($response)
    {
        if ($this->callback["hasErrors"]) {
            unset($this->callback["hasErrors"]);
            $callback = array(
                "sucesso" => false,
                "erros" => $this->callback,
            );
        } else {
            $callback = array(
                "sucesso" => true,
            );
        }

        $response->addDataset(new DataSet('status', $callback));
    }

    function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = substr($request->getRoutePath(), 1);

            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            $page = $filterCriteria->getPage();
            $pageSize = $filterCriteria->getPageSize();

            $dataSet = $this->crudService->getFilteredDataSet($this->dataSourceName, $conditions, $page, $pageSize);

            $responseDataSet = new DataSet($this->dataSourceName, $dataSet->getRows());
            $response->setDataSets([$responseDataSet]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}