<?php
namespace BackOfficeOptimizer\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use BackOfficeOptimizer\Libraries\JWT;

class Plates{
    protected $loginProvider;
    protected $sheetImporter;

    public function __construct($loginProvider, $sheetImporter){
        $this->loginProvider = $loginProvider;
        $this->sheetImporter = $sheetImporter;
    }

    public function importSheet(DTO\Request $request, DTO\Response $response) {
        try {
            $token = isset($request->getParameters()['$token'])? $request->getParameters()['$token']: null;
            if($token){
                $userInfo = JWT::getTokenData($token);
            }
            $nrorg = $userInfo['NRORG'] ? $userInfo['NRORG'] : 1;
            $row = $request->getRow();
            $sheetPath = $this->sheetImporter->createTemporaryFile($row["sheetImport"]);
            $worksheet = $this->sheetImporter->loadSheet($sheetPath);
            $sheet = $this->sheetImporter->parseSheet($worksheet);

            $this->sheetImporter->validateSheetColumns($sheet);
            $this->sheetImporter->validateSheetRows($sheet, $row['CDFILIAL'],$row['CDSERVFILI'],$row['IDTIPORESTRICAO'], $response);

            $result = $this->sheetImporter->saveSheetData($sheet, $response, $row, $nrorg);
            $nrimpparampratfilial = $result['nrimpparampratfilial'];
            $cdfilial = $result['cdfilial'];
            $cdservfili = $result['cdservfili'];
            $idtiporestricao = $result['idtiporestricao'];
            $idutpadca = $row['IDUTRPADCA'];
            $nrlogimparampratfi = $this->sheetImporter->executeProcedure($nrimpparampratfilial, $cdfilial, $cdservfili, $idtiporestricao, $idutpadca, $response);

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));

        } finally {

            if (isset($sheetPath)) {
                $this->sheetImporter->removeTemporaryFile($sheetPath);
            }

            if (isset($nrimpparampratfilial)) {
                $this->sheetImporter->removeSheetTmpData($nrimpparampratfilial);
            }

            $nrlogimparampratfi = $this->sheetImporter->getNrlogimparampratfi();
            if ($nrlogimparampratfi) {
                $this->sheetImporter->removeSheetLog($nrlogimparampratfi);
            }
        }
    }
    
}
